import { default as Axios, AxiosInstance } from 'axios';
import { ApiToken, LoginSubmit, Measure, Measurement, User } from '@/models';
import { Logger, logService } from '@/services/logging';
import merge from 'lodash.merge';

export class PmApiClient {
  private http: AxiosInstance;
  private authToken?: string;
  private log: Logger;

  constructor(apiBase: string) {
    this.http = Axios.create({
      baseURL: apiBase
      /*
      headers: {
        post: {
          'Content-Type': 'application/json'
        }
      }
      */
    });

    this.log = logService.getLogger('services/pm-api-client');
    this.log.trace('Initialized PmApiClient');
  }

  public setAuthToken(t: string) { this.authToken = t; }

  public async version(): Promise<string> {
    const resp = await this.http.get('/version');
    return resp.data;
  }

  public async createAuthToken(creds: LoginSubmit)
    : Promise<string> {

    const resp = await this.http.post('/auth-token', creds);
    return resp.data;
  }

  public async getUser(authToken: string): Promise<User> {

    const resp = await this.http.get('/user',
      { headers: { Authorization: 'Bearer ' + authToken }});

    return merge(resp.data, { authToken }) ;
  }

  public async getAllUsers(): Promise<User[]> {
    const resp = await this.http.get('/users',
      { headers: this.authHeader() });

    return resp.data;
  }

  public async getUserById(reqUserId: string): Promise<User> {
    const resp = await this.http.get(`/users/${reqUserId}`,
      { headers: this.authHeader() });

    return resp.data;
  }

  public async createUser(newUser: User): Promise<User> {
    const resp = await this.http.post('/users',
       newUser, { headers: this.authHeader() });

    return resp.data;
  }

  public async deleteUser(toDeleteUserId: string)
    : Promise<boolean> {

    await this.http.delete(`/users/${toDeleteUserId}`,
      { headers: this.authHeader() });

    return true;
  }

  public async changePwd(oldPassword: string, newPassword: string)
    : Promise<boolean> {

    await this.http.post('/change-pwd',
      { oldPassword, newPassword }, { headers: this.authHeader() });

    return true;
  }

  public async changePwdForUser(forUserId: string, newPassword: string)
    : Promise<boolean> {

    await this.http.post(`/change-pwd/${forUserId}`,
      { newPassword }, { headers: this.authHeader() });

    return true;
  }

  public async createApiToken(token: ApiToken)
    : Promise<ApiToken[]> {

    const resp = await this.http.post(`/api-tokens`,
      token, { headers: this.authHeader() });

    return resp.data;
  }

  public async getAllApiTokens(): Promise<ApiToken[]> {
    const resp = await this.http.get('/api-tokens',
      { headers: this.authHeader() });

    return resp.data;
  }

  public async getApiToken(tokenId: string): Promise<ApiToken[]> {
    const resp = await this.http.get(`/api-tokens/${tokenId}`,
      { headers: this.authHeader() });

    return resp.data;
  }

  public async deleteApiToken(tokenId: string): Promise<boolean> {
    const resp = await this.http.delete(`/api-tokens/${tokenId}`,
      { headers: this.authHeader() });

    return true;
  }

  public async getAllMeasures(): Promise<Measure[]> {
    const resp = await this.http.get(`/measures`,
      { headers: this.authHeader() });

    return resp.data;
  }

  public async createMeasure(measure: Measure): Promise<Measure> {
    const resp = await this.http.post(`/measures`,
      measure, { headers: this.authHeader() });

    return resp.data;
  }

  public async getMeasure(slug: string): Promise<Measure> {
    const resp = await this.http.get(`/measures/${slug}`,
      { headers: this.authHeader() });

    return resp.data;
  }

  public async deleteMeasure(slug: string): Promise<boolean> {
    const resp = await this.http.delete(`/measures/${slug}`,
      { headers: this.authHeader() });

    return true;
  }

  public async getMeasurements(measureSlug: string)
    : Promise<Measurement[]> {

    const resp = await this.http.get(`/measure/${measureSlug}`,
      { headers: this.authHeader() });

    return resp.data;
  }

  public async createMeasurement(
      measureSlug: string,
      measurement: Measurement)
    : Promise<Measurement> {

     const resp = await this.http.post(`/measure/${measureSlug}`,
       measurement, { headers: this.authHeader() });

     return resp.data;
  }

  public async getMeasurement(
      measureSlug: string,
      measurementId: string)
    : Promise<Measurement> {

    const resp = await this.http.get(`/measure/${measureSlug}/${measurementId}`,
      { headers: this.authHeader() });

    return resp.data;
  }

  public async updateMeasurement(
      measureSlug: string,
      measurement: Measurement)
    : Promise<Measurement> {

    const resp = await this.http.put(`/measure/${measureSlug}/${measurement.id}`,
      measurement, { headers: this.authHeader() });

    return resp.data;
  }

  public async deleteMeasurement(
      measureSlug: string,
      measurementId: string)
    : Promise<boolean> {

    const resp = await this.http.delete(`/measure/${measureSlug}/${measurementId}`,
      {headers: this.authHeader() });

    return true;
  }

  private authHeader(): { [key: string]: string } {
    if (this.authToken) {
      return { Authorization: 'Bearer ' + this.authToken };
    } else {
      throw new Error('no authenticated user');
    }
  }
}

export const api = new PmApiClient(process.env.VUE_APP_PM_API_BASE);
export default api;
